// file: i2cLCD.c
// *     RA5 SDA
// *     RA4 SCL
#include <xc.h>
#include "i2cLCD.h"

#include <stdio.h>

//******************************************************
// LCD で printf関数が使用できるようにするため putch を設定する
//******************************************************
void putch(char data) {
        LCD_dat(data);      // LCD への一文字表示関数
}

//********************************************************************
// I2C 関連
//********************************************************************
void i2c_MasterInit(const unsigned long baud){
    // pin設定 ------------------------------------------------
    TRISA5 = 1;           // RA5を入力
    TRISA4 = 1;           // RA4を入力
    ANSA5  = 0;           // RA5をデジタル
    ANSA4  = 0;           // RA4をデジタル
    ODCA5  = 1;           // RA5をオープンドレイン
    ODCA4  = 1;           // RA4をオープンドレイン
    WPUA5  = 1;
    WPUA4  = 1;

    // PPS設定　-----------------------------------------------
    SSP1CLKPPS = 0x04;    // RA4をCLK入力に指定
    RA4PPS     = 24;      // RA4をCLK出力に指定
    SSP1DATPPS = 0x05;    // RA5をDATに入力指定
    RA5PPS     = 25;      // RA5をDAT出力に指定

    // SSP1設定 -----------------------------------------------
    SSP1STAT = 0b10000000;     // スルーレート制御はOff
    SSP1ADD = (_XTAL_FREQ/(4*baud))-1;      //  SSP1ADD = 0x77;
    SSP1CON1 = 0b00101000;     // I2C Master modeにする
}

char i2c_Write(char data){
    PIR1bits.SSP1IF = 0;           // 終了フラグクリア
    SSP1BUF = data;                // データセット
    while(!PIR1bits.SSP1IF);       // 送信終了待ち
    return SSP1CON2bits.ACKSTAT;   // ACKなら「0」で復帰
}                                  // NACKは「1」で復帰

//-------- ByteI2C送信
void i2cByteWrite(char addr, char cont, char data){
    i2c_Idle();                // Bus アイドル確認
    i2c_Start();               // Start bit送信
    i2c_Write(addr);           // アドレス送信
    i2c_Write(cont);           // 制御コード送信
    i2c_Write(data);           // データ送信
    i2c_Stop();                // Stop bit送信
}

//********************************************************************
// LCD 関連
//********************************************************************
//-------- １文字表示
void LCD_dat(char chr){
    i2cByteWrite(0x7C, 0x40, chr);
    __delay_us(50);            // 50μsec
}
//-------- コマンド出力
void LCD_cmd(char cmd){
    i2cByteWrite(0x7C, 0x00, cmd);
    if(cmd & 0xFC)             // 上位６ビットに１がある命令
        __delay_us(50);        // 50usec
    else
        __delay_ms(2);         // 2msec ClearおよびHomeコマンド
}
//-------- 全消去
void LCD_clr(void){
    LCD_cmd(0x01);             //Clearコマンド出力
}
//-------- カーソル位置指定
void LCD_posyx(char ypos, char xpos){
    unsigned char pcode;
    switch(ypos & 0x03){
        case 0:    pcode=0x80;break;
        case 1:    pcode=0xC0;break;
    }
    LCD_cmd(pcode += xpos);
}
//-------- 初期化
void LCD_int(void){
    __delay_ms(100);
    LCD_cmd(0x38);             // 8bit 2行 表示命令モード
    LCD_cmd(0x39);             // 8bit 2行 拡張命令モード
    LCD_cmd(0x14);             // OSC  BIAS 設定1/5
                               // コントラスト設定
    LCD_cmd(0x70 + (CONTRAST & 0x0F));
    LCD_cmd(0x5C + (CONTRAST >> 4));
    LCD_cmd(0x6B);             // Ffollwer
    __delay_ms(100);
    __delay_ms(100);
    LCD_cmd(0x38);             // 表示命令モード
    LCD_cmd(0x0C);             // Display On
    LCD_cmd(0x01);             // Clear Display
}
//-------- 文字列出力
void LCD_str(char *str){
    while(*str)                //文字列の終わり(00)まで継続
        LCD_dat(*str++);       //文字出力しポインタ＋１
}
//-------- Rom 文字列出力
void LCD_ROMstr(const char *str){
    while(*str)                //文字列の終わり(00)まで継続
        LCD_dat(*str++);       //文字出力しポインタ＋１
}

